searchData={"items":[{"type":"module","title":"Mneme","doc":"Augments `ExUnit.Assertions` with a set of assertions that know how to\nupdate themselves.\n\n> #### `use Mneme` {: .info}\n>\n> When you `use Mneme` in a test module, [assertions](#assertions) are\n> imported and module attributes are made available for\n> [configuration](#configuration).","ref":"Mneme.html"},{"type":"module","title":"Configuration - Mneme","doc":"Mneme supports a variety of flexible configuration options that can be\napplied at multiple levels of granularity, from your entire test suite\ndown to an individual test. While the default behavior will work well\nfor the majority of cases, it's worth knowing which levers and knobs\nyou have available to tweak Mneme to fit your individual workflow.\n\n#","ref":"Mneme.html#module-configuration"},{"type":"module","title":"Options - Mneme","doc":"* `:action` (`:prompt | :accept | :reject`) - The action to be taken when an auto-assertion updates. If `CI=true`\n  is set in environment variables, the action will _always_ be\n  `:reject`. The default value is `:prompt`.\n\n* `:default_pattern` (`:infer | :first | :last`) - The default pattern to be selected if prompted to update an assertion. The default value is `:infer`.\n\n* `:diff` (`:text | :semantic`) - Controls the diff engine used to display changes when an auto-\n  assertion updates. If `:semantic`, uses a custom diff engine to\n  highlight only meaningful changes in the value. If `:text`, uses\n  the Myers Difference algorithm to highlight all changes in text. The default value is `:semantic`.\n\n* `:diff_style` (`:side_by_side | :stacked`) - Controls how diffs are rendered when the `:diff` option is set to\n  `:semantic`. If `:side_by_side`, old and new code will be rendered\n  side-by-side if the terminal has sufficient space. If `:stacked`,\n  old and new code will be rendered one on top of the other. The default value is `:side_by_side`.\n\n* `:force_update` (`t:boolean/0`) - Setting to `true` will force auto-assertions to update even when\n  they would otherwise succeed. This can be especially helpful when\n  adding new keys to maps or structs since a pattern like `%{}`\n  would not normally prompt as the match still succeeds. The default value is `false`.\n\n* `:target` (`:mneme | :ex_unit`) - The target output for auto-assertions. If `:mneme`, the expression\n  will remain an auto-assertion. If `:ex_unit`, the expression will\n  be rewritten as an ExUnit assertion. The default value is `:mneme`.\n\n\n\n#","ref":"Mneme.html#module-options"},{"type":"module","title":"Configuring Mneme - Mneme","doc":"There are four ways you can apply configuration options; Each is more\nspecific than the last and will override any conflicting options that\nwere set prior.\n\n  * When calling `Mneme.start/1`, which will apply to the entire test\n    run;\n  * When calling `use Mneme`, which will apply to all tests in that\n    module;\n  * In a `@mneme_describe` module attribute, which will apply to all\n    tests that follow in the given `ExUnit.Case.describe/2` block;\n  * In a `@mneme` module attribute, which will apply only to the next\n    test that follows.\n\nFor instance, when an auto-assertion has multiple possible patterns\navailable, Mneme will try to infer the best one to show you first.\nIf you always want the last (and usually most complex) generated\npattern, you could call `Mneme.start/1` like this:\n\n    # test/test_helper.exs\n    ExUnit.start()\n    Mneme.start(default_pattern: :last)\n\nAs mentioned above, this can be overriden at the module-level, in a\n`describe` block, or for an individual test:\n\n    defmodule MyTest do\n      use ExUnit.Case\n      use Mneme, default_pattern: :infer\n\n      test \"the default pattern will exclude :baz when this runs\" do\n        map = %{foo: :one, baz: :three}\n        auto_assert %{foo: 1, bar: 2} <- Map.put(map, bar: :two)\n      end\n\n      describe \"...\" do\n        @mneme_describe action: :reject\n\n        test \"fails without prompting\" do\n          auto_assert :wrong <- MyModule.some_fun()\n        end\n\n        @mneme action: :prompt\n        test \"prompts to update\" do\n          auto_assert :wrong <- MyModule.another_fun()\n        end\n      end\n    end\n\nBreaking up with Mneme? While the official stance of the library is\nthat this is Not Recommended™, Mneme can even convert all of your\nexisting auto-assertions to regular assertions:\n\n    Mneme.start(\n      target: :ex_unit,\n      force_update: true,\n      action: :accept\n    )\n\nProbably don't do this, but if you do, make sure all your tests are\ncommitted first in case you want to get back together.","ref":"Mneme.html#module-configuring-mneme"},{"type":"macro","title":"Mneme.auto_assert/1","doc":"Pattern-generating variant of `ExUnit.Assertions.assert/1`.","ref":"Mneme.html#auto_assert/1"},{"type":"macro","title":"Examples - Mneme.auto_assert/1","doc":"`auto_assert` generates assertions when tests run, issuing a terminal\nprompt before making any changes (unless configured otherwise).\n\n    auto_assert [1, 2] ++ [3, 4]\n\n    # after running the test and accepting the change\n    auto_assert [1, 2, 3, 4] <- [1, 2] ++ [3, 4]\n\nIf the match no longer succeeds, a warning and new prompt will be\nissued to update it to the new value.\n\n    auto_assert [1, 2, 3, 4] <- [1, 2] ++ [:a, :b]\n\n    # after running the test and accepting the change\n    auto_assert [1, 2, :a, :b] <- [1, 2] ++ [:a, :b]\n\nPrompts are only issued if the pattern doesn't match the value, so\nthat pattern can also be changed manually.\n\n    # this assertion succeeds, so no prompt is issued\n    auto_assert [1, 2, | _] <- [1, 2] ++ [:a, :b]","ref":"Mneme.html#auto_assert/1-examples"},{"type":"macro","title":"Differences from ExUnit `assert` - Mneme.auto_assert/1","doc":"The `auto_assert` macro is meant to match `assert` very closely, but\nthere are a few differences to note:\n\n  * Pattern-matching assertions use the `<-` operator instead of the\n    `=` match operator.\n\n  * Unlike ExUnit's `assert`, `auto_assert` can match falsy values.\n    The following are equivalent:\n\n        falsy = nil\n        auto_assert nil <- falsy\n        assert falsy == nil\n\n  * Guards can be added with a `when` clause, while `assert` would\n    require a second assertion. For example:\n\n        auto_assert pid when is_pid(pid) <- self()\n\n        assert pid = self()\n        assert is_pid(pid)\n\n  * Bindings in an `auto_assert` are not available outside of that\n    assertion. For example:\n\n        auto_assert pid when is_pid(pid) <- self()\n        pid # ERROR: pid is not bound\n\n    If you need to use the result of the assertion, it will evaluate\n    to the expression's value.\n\n        pid = auto_assert pid when is_pid(pid) <- self()\n        pid # pid is the result of self()","ref":"Mneme.html#auto_assert/1-differences-from-exunit-assert"},{"type":"macro","title":"Mneme.auto_assert_raise/1","doc":"See `auto_assert_raise/3`.","ref":"Mneme.html#auto_assert_raise/1"},{"type":"macro","title":"Mneme.auto_assert_raise/2","doc":"See `auto_assert_raise/3`.","ref":"Mneme.html#auto_assert_raise/2"},{"type":"macro","title":"Mneme.auto_assert_raise/3","doc":"Pattern-generating variant of `ExUnit.Assertions.assert_raise/3`.\n\nIf the given function does not raise, the assertion will fail.\n\nLike `auto_assert/1`, you will be prompted to automatically update\nthe assertion if the raised raised exception changes.","ref":"Mneme.html#auto_assert_raise/3"},{"type":"macro","title":"Examples - Mneme.auto_assert_raise/3","doc":"You can pass an anonymous function that takes no arguments and is\nexpected to raise an exception.\n\n    auto_assert_raise fn ->\n      some_call_expected_to_raise()\n    end\n\n    # after running the test and accepting changes\n    auto_assert_raise Some.Exception, fn ->\n      some_call_expected_to_raise()\n    end\n\n    # optionally include the message\n    auto_assert_raise Some.Exception, \"perhaps with a message\", fn ->\n      some_call_expected_to_raise()\n    end\n\nA captured function of arity zero can also be used.\n\n    auto_assert_raise &some_call_expected_to_raise/0\n\n    # after running the test and accepting changes\n    auto_assert_raise Some.Exception, &some_call_expected_to_raise/0","ref":"Mneme.html#auto_assert_raise/3-examples"},{"type":"macro","title":"Mneme.auto_assert_receive/0","doc":"See `auto_assert_receive/2`.","ref":"Mneme.html#auto_assert_receive/0"},{"type":"macro","title":"Mneme.auto_assert_receive/1","doc":"See `auto_assert_receive/2`.","ref":"Mneme.html#auto_assert_receive/1"},{"type":"macro","title":"Mneme.auto_assert_receive/2","doc":"Pattern-generating variant of `ExUnit.Assertions.assert_receive/3`.\n\n`timeout` is in milliseconds and defaults to `100`.","ref":"Mneme.html#auto_assert_receive/2"},{"type":"macro","title":"Examples - Mneme.auto_assert_receive/2","doc":"Process.send_after(self(), {:some, :message}, 50)\n\n    auto_assert_receive()\n\n    # after running the test, messages appearing within 100ms\n    # will be available as options\n    auto_assert_receive {:some, :message}\n\nA custom timeout can be specified as a second argument.\n\n    Process.send_after(self(), {:some, :message}, 150)\n\n    auto_assert_receive nil, 300\n\n    # messages appearing within 300ms will now appear as options\n    auto_assert_receive {:some, :message}, 300","ref":"Mneme.html#auto_assert_receive/2-examples"},{"type":"macro","title":"Mneme.auto_assert_received/0","doc":"See `auto_assert_received/1`.","ref":"Mneme.html#auto_assert_received/0"},{"type":"macro","title":"Mneme.auto_assert_received/1","doc":"Pattern-generating variant of `ExUnit.Assertions.assert_received/2`.\n\nSimilar to `auto_assert_receive/2`, except that the timeout is set to\n0, so the expected message must already be in the current process'\nmailbox.","ref":"Mneme.html#auto_assert_received/1"},{"type":"macro","title":"Examples - Mneme.auto_assert_received/1","doc":"send(self(), {:some, :message})\n\n    auto_assert_received()\n\n    # after running the test, messages in the current process\n    # inbox will be available as options\n    auto_assert_receive {:some, :message}","ref":"Mneme.html#auto_assert_received/1-examples"},{"type":"function","title":"Mneme.start/1","doc":"Starts Mneme to run auto-assertions as they appear in your tests.\n\nThis will almost always be added to your `test/test_helper.exs`, just\nbelow the call to `ExUnit.start()`:\n\n    # test/test_helper.exs\n    ExUnit.start()\n    Mneme.start()","ref":"Mneme.html#start/1"},{"type":"function","title":"Options - Mneme.start/1","doc":"* `:restart` (boolean) - Restarts Mneme if it has previously been\n    started. This option enables certain IEx-based testing workflows\n    that allow tests to be run without a startup penalty. Defaults to\n    `false`.\n\n  * Any option defined in the [Configuration](#module-configuration)\n    section of the module docs.","ref":"Mneme.html#start/1-options"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nThis format is based on [Keep a Changelog](https://keepachangelog.com) and this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).","ref":"changelog.html"},{"type":"extras","title":"v0.4.3 (2023) - Changelog","doc":"#","ref":"changelog.html#v0-4-3-2023"},{"type":"extras","title":"Fixes - Changelog","doc":"* Fix a regression introduced in v0.4.1 that caused auto-assertions to fail if an earlier one in the same file added or removed lines when formatted ([#53](https://github.com/zachallaun/mneme/issues/53)).\n  * Suppress \"unused alias\" warnings when the aliases are only used in auto-assertions ([#54](https://github.com/zachallaun/mneme/issues/54)).","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.4.2 (2023-11-01) - Changelog","doc":"#","ref":"changelog.html#v0-4-2-2023-11-01"},{"type":"extras","title":"Fixes - Changelog","doc":"* Fix a crash that would occur if a test file containing an auto-assertion is changed before the auto-assertion is run (for instance, while Mneme is waiting on input from an auto-assertion in a different test file).","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.4.1 (2023-10-25) - Changelog","doc":"#","ref":"changelog.html#v0-4-1-2023-10-25"},{"type":"extras","title":"Fixed - Changelog","doc":"* Fix a crash that could occur when diffing sigils ([#52](https://github.com/zachallaun/mneme/issues/52)).","ref":"changelog.html#fixed"},{"type":"extras","title":"v0.4.0 (2023-07-04) - Changelog","doc":"#","ref":"changelog.html#v0-4-0-2023-07-04"},{"type":"extras","title":"Added - Changelog","doc":"* `Mneme.start/1` now accepts all of Mneme's configuration options that will be applied to the entire test run.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Removed - Changelog","doc":"* **Breaking:** Removed support for `:mneme` application config. This has been replaced by passing config options directly to `Mneme.start/1`:\n\n      ```elixir\n      # Old: config/test.exs\n      config :mneme, defaults: [default_pattern: :last, ...]\n\n      # New: test/test_helper.exs\n      Mneme.start(default_pattern: :last, ...)\n      ```","ref":"changelog.html#removed"},{"type":"extras","title":"v0.3.5 (2023-06-30) - Changelog","doc":"#","ref":"changelog.html#v0-3-5-2023-06-30"},{"type":"extras","title":"Fixed - Changelog","doc":"* Support multi-letter sigils when using Elixir 1.15.0+.","ref":"changelog.html#fixed"},{"type":"extras","title":"v0.3.4 (2023-05-22) - Changelog","doc":"#","ref":"changelog.html#v0-3-4-2023-05-22"},{"type":"extras","title":"Added - Changelog","doc":"* Tested to support OTP 26.0 when using Elixir 1.14.4.\n  * Support expressions that return functions, serializing them with an `is_function(fun, arity)` guard.\n  * When generating a pattern for a MapSet, add a note suggesting using `MapSet.to_list/1` for better serialization.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"* Format pattern notes to be more obvious when they're present.\n  * Generate charlist patterns using `sigil_c` instead of single quotes, e.g. `~c\"foo\"` instead of `'foo'`. See [this discussion](https://elixirforum.com/t/convert-charlists-into-c-charlists/49455) for more context.\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"* Numerous fixes related to vars used in guards:\n    * Generated vars will no longer shadow variables in scope (e.g. if `pid` is in scope, a different pid will use the var `pid1`).\n    * The same var will no longer be used for different values of the same type.\n    * Multiple, redundant guards will no longer be emitted for the same var (e.g. `[self(), self()]` would result in `[pid, pid] when is_pid(pid) and is_pid(pid)`).\n  * Numerous fixes related to pattern generation, especially in regards to map keys.","ref":"changelog.html#fixed"},{"type":"extras","title":"v0.3.3 (2023-05-01) - Changelog","doc":"#","ref":"changelog.html#v0-3-3-2023-05-01"},{"type":"extras","title":"Changed - Changelog","doc":"* Improve stacktraces and ExUnit assertion errors when an auto-assertion fails or is rejected.\n  * When an `auto_assert` updates, a default pattern that is more similar to the existing one will be selected in more cases when the `:default_pattern` option is set to `:infer` (the default).\n  * When updating an assertion with an existing map pattern that only asserts a subset of keys, generate a pattern using that subset as well.","ref":"changelog.html#changed"},{"type":"extras","title":"v0.3.2 (2023-04-16) - Changelog","doc":"#","ref":"changelog.html#v0-3-2-2023-04-16"},{"type":"extras","title":"Changed - Changelog","doc":"* Multi-line string patterns will now always appear in the same order (heredoc format will always be the last option).\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"* Raise a more comprehensible error if `Mneme.start()` is called multiple times without `restart: true`.\n  * Fix an incorrect guard that could cause semantic diffing to fail and fall back to text diffing.\n  * Fix multi-line string formatting issues with `auto_assert_raise`, `auto_assert_receive`, and `auto_assert_received`.","ref":"changelog.html#fixed"},{"type":"extras","title":"v0.3.1 (2023-04-14) - Changelog","doc":"#","ref":"changelog.html#v0-3-1-2023-04-14"},{"type":"extras","title":"Changed - Changelog","doc":"* Multi-line strings will now generate both a heredoc and a single-line option.\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Removed - Changelog","doc":"* No longer depend on `libgraph`.","ref":"changelog.html#removed"},{"type":"extras","title":"v0.3.0 (2023-04-10) - Changelog","doc":"It is recommended to now use Elixir v1.14.4 or later.\n\n#","ref":"changelog.html#v0-3-0-2023-04-10"},{"type":"extras","title":"Added - Changelog","doc":"* Add three new auto-assertions:\n    * [`auto_assert_raise`](https://hexdocs.pm/mneme/Mneme.html#auto_assert_raise/3)\n    * [`auto_assert_receive`](https://hexdocs.pm/mneme/Mneme.html#auto_assert_receive/2)\n    * [`auto_assert_received`](https://hexdocs.pm/mneme/Mneme.html#auto_assert_received/1)\n  * Add a `:force_update` option that forces re-generating assertion patterns even when they succeed. See the [Options documentation](https://hexdocs.pm/mneme/Mneme.html#module-options) for more info.\n  * Prompts will now show options that were overridden from the defaults.\n  * Mneme now prints a one-line summary at the end of the test run.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"* For falsy values, `auto_assert` now generates `<-` pattern matches instead of `==` value comparisons, which have been removed.\n  * Existing auto-assertions will now run when `Mneme.start()` is not called, but new or failing auto-assertions will fail without prompting ([#32](https://github.com/zachallaun/mneme/issues/32)).\n  * Ranges now use range syntax like `1..10` and `1..10//2` instead of generating a `%Range{}` struct.\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Removed - Changelog","doc":"* **Breaking:** `auto_assert` no longer supports value comparisons using `==`.\n\n#","ref":"changelog.html#removed"},{"type":"extras","title":"Fixed - Changelog","doc":"* Fix a configuration precedence bug that caused options set in application config to always override module, describe, or test options.\n  * Fix a compatibility issue with Ecto ~> 3.9.4 ([#34](https://github.com/zachallaun/mneme/issues/34)).\n  * Fix a confusing diff result that could occur with some binary operations ([#11](https://github.com/zachallaun/mneme/issues/11)).\n  * Preceding comments are no longer shown in diffs ([#26](https://github.com/zachallaun/mneme/issues/26)).\n  * Fix a number of diffing errors related to structs.","ref":"changelog.html#fixed"},{"type":"extras","title":"v0.2.7 (2023-03-29) - Changelog","doc":"#","ref":"changelog.html#v0-2-7-2023-03-29"},{"type":"extras","title":"Fixed - Changelog","doc":"* Fix a crash related to escaped string interpolation characters ([#29](https://github.com/zachallaun/mneme/issues/29)).","ref":"changelog.html#fixed"},{"type":"extras","title":"v0.2.6 (2023-03-27) - Changelog","doc":"#","ref":"changelog.html#v0-2-6-2023-03-27"},{"type":"extras","title":"Added - Changelog","doc":"* Auto-assertion prompts can now be skipped (`s`) in addition to accepted (`y`) or rejected (`n`). This allows the test clause to continue so that later assertions might be run, but fails the test run once the suite finishes.\n  * Semantic diffs will now be displayed side-by-side if terminal width allows. To always display diffs stacked, use the `diff_style: :stacked` option; see the \"Configuration\" section of the `Mneme` module doc for more.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"* Semantic diff formatting has been improved for clarity.\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"* Don't overwrite test files if their content changes after starting the test run ([#23](https://github.com/zachallaun/mneme/issues/23)).\n  * Fix a crash that occurred when a value contained nested strings with newlines, e.g. `{:ok, \"hello\\nworld\"}` ([#25](https://github.com/zachallaun/mneme/issues/25)).\n  * The `j/k` options will no longer be rendered when prompting if there is only a single pattern option.","ref":"changelog.html#fixed"},{"type":"extras","title":"v0.2.4, v0.2.5 (2023-03-25) - Changelog","doc":"#","ref":"changelog.html#v0-2-4-v0-2-5-2023-03-25"},{"type":"extras","title":"Fixed - Changelog","doc":"* Remove unnecessary files from Hex package. This cuts the package size down drastically.","ref":"changelog.html#fixed"},{"type":"extras","title":"v0.2.3 (2023-03-25) - Changelog","doc":"#","ref":"changelog.html#v0-2-3-2023-03-25"},{"type":"extras","title":"Fixed - Changelog","doc":"* Fix diffing for certain sigil variations.\n  * Fix `dbg`-related error when running `MIX_ENV=test iex -S mix` ([#20](https://github.com/zachallaun/mneme/issues/20)).\n  * Fix ETS-related error when calling `Mneme.start/1` multiple times ([#20](https://github.com/zachallaun/mneme/issues/20#issuecomment-1483878101)).","ref":"changelog.html#fixed"},{"type":"extras","title":"v0.2.2 (2023-03-20) - Changelog","doc":"#","ref":"changelog.html#v0-2-2-2023-03-20"},{"type":"extras","title":"Fixed - Changelog","doc":"* Disable a semantic diffing optimization that caused poor diff results in certain cases, usually manifesting as incorrect branches being compared.","ref":"changelog.html#fixed"},{"type":"extras","title":"v0.2.1 (2023-03-19) - Changelog","doc":"#","ref":"changelog.html#v0-2-1-2023-03-19"},{"type":"extras","title":"Changed - Changelog","doc":"* More consistent formatting between `:semantic` and `:text` diffs.","ref":"changelog.html#changed"},{"type":"extras","title":"v0.2.0 (2023-03-18) - Changelog","doc":"#","ref":"changelog.html#v0-2-0-2023-03-18"},{"type":"extras","title":"Added - Changelog","doc":"* Adds semantic diffs which selectively highlight only meaningful changes when updating an assertion. This can be disabled with the `diff: :text` option; see the \"Configuration\" section of the `Mneme` module doc for more.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"* **Breaking:** Mneme now requires Elixir v1.14 or later.\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"* Invalid options now cause a warning instead of crashing test process.\n  * Internal errors now show an error instead of crashing test process.\n  * Fix bug causing multiple identical choices to be presented in some cases where empty lists were a part of the value.","ref":"changelog.html#fixed"},{"type":"extras","title":"v0.1.6 (2023-03-04) - Changelog","doc":"#","ref":"changelog.html#v0-1-6-2023-03-04"},{"type":"extras","title":"Changed - Changelog","doc":"* Improved compile-time error message when `auto_assert` is used outside of a `test` block ([#9](https://github.com/zachallaun/mneme/issues/9)).","ref":"changelog.html#changed"},{"type":"extras","title":"v0.1.5 (2023-02-25) - Changelog","doc":"#","ref":"changelog.html#v0-1-5-2023-02-25"},{"type":"extras","title":"Changed - Changelog","doc":"* More consistent handling of charlists: lists of integers will now generate themselves as a pattern as well as a charlist if the list is ASCII printable ([#6](https://github.com/zachallaun/mneme/issues/6)).","ref":"changelog.html#changed"},{"type":"extras","title":"v0.1.4 (2023-02-23) - Changelog","doc":"#","ref":"changelog.html#v0-1-4-2023-02-23"},{"type":"extras","title":"Fixed - Changelog","doc":"* Fix a bug that could cause `auto_assert` expressions to revert to the previous value when using `Mneme.start(restart: true)` ([#7](https://github.com/zachallaun/mneme/issues/7)).","ref":"changelog.html#fixed"},{"type":"extras","title":"v0.1.3 (2023-02-22) - Changelog","doc":"#","ref":"changelog.html#v0-1-3-2023-02-22"},{"type":"extras","title":"Added - Changelog","doc":"* Add a `:default_pattern` configuration option for auto-assertions which controls the pattern that should be selected by default when prompted.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"* When converting an auto-assertion to an ExUnit assertion, select the identical pattern when the `:default_pattern` is `:infer` (set by default).","ref":"changelog.html#fixed"},{"type":"extras","title":"v0.1.2 (2023-02-21) - Changelog","doc":"#","ref":"changelog.html#v0-1-2-2023-02-21"},{"type":"extras","title":"Added - Changelog","doc":"* Add a `:restart` option to `Mneme.start/1` to restart Mneme if called multiple times.","ref":"changelog.html#added"},{"type":"extras","title":"v0.1.1 (2023-02-20) - Changelog","doc":"#","ref":"changelog.html#v0-1-1-2023-02-20"},{"type":"extras","title":"Changed - Changelog","doc":"* Dramatically reduce the performance gap between `auto_assert` and ExUnit's `assert`.","ref":"changelog.html#changed"},{"type":"extras","title":"v0.1.0 (2023-02-19) - Changelog","doc":"First release.","ref":"changelog.html#v0-1-0-2023-02-19"},{"type":"extras","title":"Overview","doc":"# /ˈniːmiː/ - Snapshot testing utilities for Elixir\n\n[![Hex.pm](https://img.shields.io/hexpm/v/mneme.svg)](https://hex.pm/packages/mneme)\n[![Docs](https://img.shields.io/badge/hexdocs-docs-8e7ce6.svg)](https://hexdocs.pm/mneme)\n[![CI](https://github.com/zachallaun/mneme/actions/workflows/ci.yml/badge.svg?branch=main)](https://github.com/zachallaun/mneme/actions/workflows/ci.yml)\n\n \n   🎥 Video Demo \n   https://user-images.githubusercontent.com/503938/227819477-c7097fbc-b9a4-44a1-b3ea-f1b420c18799.mp4 \n \n\nMneme augments `ExUnit.Assertions` with a set of assertions that know how to update themselves.\nThis is sometimes called snapshot or approval testing.\n\nWith Mneme, you write something like...\n\n```elixir\nauto_assert my_function()\n```\n\n...and next time you run your tests, Mneme:\n\n1. runs the code and generates a pattern from the returned value, then\n2. prints a diff and prompts you to confirm the change.\n\nNow your test looks like this:\n\n```elixir\nauto_assert %MyAwesomeValue{so: :cool} <- my_function()\n```\n\nThis lets you quickly write lots of tests, and like ordinary tests, you'll see a diff when they fail.\nBut, unlike ordinary tests, you can choose to accept the changes and Mneme will rewrite the test for you.\n\n**Features:**\n\n  * **Auto-updating assertions:** maintain correct tests as behavior changes during development.\n  * **Alternatives to familiar assertions** `auto_assert`, `auto_assert_raise`, `auto_assert_receive`, `auto_assert_received`.\n  * **Seamless integration with ExUnit:** no need to change your workflow, just run `mix test`.\n  * **Interactive prompts in your terminal** when a new assertion is added or an existing one changes.\n  * **Syntax-aware diffs** highlight the meaningful changes in a value.\n\n**Take a brief tour:**\n\nIf you'd like to see Mneme in action, you can download and run [examples/tour_mneme.exs](https://github.com/zachallaun/mneme/blob/main/examples/tour_mneme.exs), a standalone tour that only requires that you have Elixir installed.\n\n```shell\n$ curl -o tour_mneme.exs https://raw.githubusercontent.com/zachallaun/mneme/main/examples/tour_mneme.exs\n\n$ elixir tour_mneme.exs\n```","ref":"readme.html"},{"type":"extras","title":"Quickstart - Overview","doc":"1.  Add `:mneme` do your deps in `mix.exs`:\n\n    ```elixir\n    defp deps do\n      [\n        {:mneme, \">= 0.0.0\", only: [:dev, :test]}\n      ]\n    end\n    ```\n\n2.  Add `:mneme` to your `:import_deps` in `.formatter.exs`:\n\n    ```elixir\n    [\n      import_deps: [:mneme],\n      inputs: [\"{mix,.formatter}.exs\", \"{config,lib,test}/**/*.{ex,exs}\"]\n    ]\n    ```\n\n3.  Start Mneme right after you start ExUnit in `test/test_helper.exs`:\n\n    ```elixir\n    ExUnit.start()\n    Mneme.start()\n    ```\n\n4.  Add `use Mneme` wherever you `use ExUnit.Case`:\n\n    ```elixir\n    defmodule MyTest do\n      use ExUnit.Case, async: true\n      use Mneme\n\n      test \"arithmetic\" do\n        auto_assert 2 + 2\n      end\n    end\n    ```\n\n5.  Run `mix test` and type `y ` when prompted; your test should look like:\n\n    ```elixir\n    defmodule MyTest do\n      use ExUnit.Case, async: true\n      use Mneme\n\n      test \"arithmetic\" do\n        auto_assert 4 <- 2 + 2\n      end\n    end\n    ```","ref":"readme.html#quickstart"},{"type":"extras","title":"Requirements - Overview","doc":"#","ref":"readme.html#requirements"},{"type":"extras","title":"Elixir - Overview","doc":"Mneme requires Elixir version 1.14 or later.\n\n#","ref":"readme.html#elixir"},{"type":"extras","title":"Formatter - Overview","doc":"**If you do not use a formatter, the first auto-assertion will reformat the entire file,** introducing unrelated formatting changes.\nMneme rewrites your test scripts when updating an assertion using the formatter configuration for your project.\n\nIt's highly recommended to configure your editor to format Elixir files on-save.\n\nNote: Mneme uses [`Rewrite`](https://github.com/hrzndhrn/rewrite) to update files, which supports the [Elixir formatter](https://hexdocs.pm/mix/Mix.Tasks.Format.html) and [`FreedomFormatter`](https://github.com/marcandre/freedom_formatter).","ref":"readme.html#formatter"},{"type":"extras","title":"Generated patterns - Overview","doc":"Mneme tries to generate match patterns that are close to what you would write.\nBasic data types like numbers, lists, tuples, and strings will generate as you would expect.\nFor maps and structs, Mneme will often give you a couple of options.\nFor values without a literal representation, like pids, guards will be used.\n\n```elixir\nauto_assert pid when is_pid(pid) <- self()\n```\n\nAdditionally, local bindings can be found and pinned as a part of patterns.\nThis keeps the number of \"magic values\" down and makes tests more robust.\n\n```elixir\ntest \"create_post/1 creates a new post with valid attrs\", %{user: user} do\n  valid_attrs = %{title: \"my_post\", author: user}\n\n  auto_assert create_post(valid_attrs)\nend\n\n# generates:\n\ntest \"create_post/1 creates a new post with valid attrs\", %{user: user} do\n  valid_attrs = %{title: \"my_post\", author: user}\n\n  auto_assert {:ok, %Post{title: \"my_post\", author: ^user}} <- create_post(valid_attrs)\nend\n```\n\n#","ref":"readme.html#generated-patterns"},{"type":"extras","title":"Special patterns - Overview","doc":"This is a non-exhaustive list of things Mneme takes into consideration when generating match patterns:\n\n  * Pinned variables are generated by default if a value is equal to a variable in scope.\n\n  * Date and time values are written using their sigil representation.\n\n  * Struct patterns only include fields that are different from the struct defaults.\n\n  * Structs defined by Ecto schemas exclude primary keys, association foreign keys, and auto generated fields like `:inserted_at` and `:updated_at`. This is because these fields are often randomly generated and would fail on subsequent tests.","ref":"readme.html#special-patterns"},{"type":"extras","title":"Continuous Integration - Overview","doc":"In a CI environment, Mneme will not attempt to prompt and update any assertions, but will instead fail any tests that would update.\nThis behavior is enabled by the `CI` environment variable, which is set by convention by many continuous integration providers.\n\n```bash\nexport CI=true\n```","ref":"readme.html#continuous-integration"},{"type":"extras","title":"Editor support - Overview","doc":"Guides for optional editor integration can be found here:\n\n  * [VS Code](https://hexdocs.pm/mneme/vscode_setup.html)","ref":"readme.html#editor-support"},{"type":"extras","title":"Acknowledgements - Overview","doc":"Special thanks to:\n\n  * [_What if writing tests was a joyful experience?_](https://blog.janestreet.com/the-joy-of-expect-tests/),\n    from the Jane Street Tech Blog, for inspiring this library.\n\n  * [Sourceror](https://github.com/doorgan/sourceror), a library that\n    makes complex code modifications simple.\n\n  * [Rewrite](https://github.com/hrzndhrn/rewrite), which provides the\n    diff functionality present in Mneme.\n\n  * [Owl](https://github.com/fuelen/owl), which makes it much easier\n    to build a pretty CLI.\n\n  * [Insta](https://insta.rs/), a snapshot testing tool for Rust,\n    whose great documentation provided an excellent reference for\n    snapshot testing.\n\n  * [assert_value](https://github.com/assert-value/assert_value_elixir),\n    an existing Elixir project that provides similar functionality.\n    Thank you for paving the way!","ref":"readme.html#acknowledgements"},{"type":"extras","title":"VS Code","doc":"# VS Code Setup\n\nIf you are using Visual Studio Code and the [ElixirLS](https://github.com/elixir-lsp/vscode-elixir-ls) extension, there are a couple of things that you can do to smooth out the workflow of running and updating your tests.\n\nThe ElixirLS extension comes with some built-in tasks to run tests:\n\n  * `mix: Run tests`\n  * `mix: Run tests in current file`\n  * `mix: Run test at cursor`\n\nThese are accessible by running the command `Run task` and then typing typing one of the tasks.\nBy default, these will open a terminal panel, but will not focus it, which isn't optimal as Mneme requires input when something changes.\n\nWe can bind these tasks to a keyboard shortcut globally and then update the focus defaults on a per-project basis.","ref":"vscode_setup.html"},{"type":"extras","title":"Keyboard shortcuts - VS Code","doc":"Run the command `Preferences: Open Keyboard Shortcuts (JSON)`, then copy the following keyboard shortcuts to the bottom of the list:\n\n```json\n{\n  \"key\": \"ctrl+space a\",\n  \"command\": \"workbench.action.tasks.runTask\",\n  \"args\": \"mix: Run tests\"\n},\n{\n  \"key\": \"ctrl+space f\",\n  \"command\": \"workbench.action.tasks.runTask\",\n  \"args\": \"mix: Run tests in current file\"\n},\n{\n  \"key\": \"ctrl+space c\",\n  \"command\": \"workbench.action.tasks.runTask\",\n  \"args\": \"mix: Run test at cursor\"\n},\n```\n\nYou can replace `\"key\"` with whatever you'd prefer.","ref":"vscode_setup.html#keyboard-shortcuts"},{"type":"extras","title":"Per-project task defaults - VS Code","doc":"Tasks provided by extensions can be modified by adding a `.vscode/tasks.json` to your project's root.\nThe following will modify the tasks listed above to give input focus to the panel they're run in.\n\n```json\n{\n  \"version\": \"2.0.0\",\n  \"tasks\": [\n    {\n      \"group\": \"test\",\n      \"type\": \"mix\",\n      \"task\": \"Run tests\",\n      \"presentation\": {\n        \"focus\": true,\n      },\n    },\n    {\n      \"group\": \"test\",\n      \"type\": \"mix\",\n      \"task\": \"Run tests in current file\",\n      \"presentation\": {\n        \"focus\": true,\n      },\n    },\n    {\n      \"group\": \"test\",\n      \"type\": \"mix\",\n      \"task\": \"Run test at cursor\",\n      \"presentation\": {\n        \"focus\": true,\n      },\n    },\n  ],\n}\n```\n\nThe Mneme repository has this set up [here](https://github.com/zachallaun/mneme/blob/main/.vscode/tasks.json), for example.","ref":"vscode_setup.html#per-project-task-defaults"},{"type":"extras","title":"Under the hood","doc":"To understand how Mneme works, we'll follow the order of execution when running `mix test`.\nAfterwards, we'll dive into some specific areas of interest.","ref":"architecture.html"},{"type":"extras","title":"Overview - Under the hood","doc":"1. `Mneme.start/1` is called first. This modifies the ExUnit configuration and starts the Mneme server, a `GenServer` that coordinates user interaction, manages IO, and stores assertion state.\n\n2. Test modules are loaded and Mneme's macros are expanded:\n    * `use Mneme` registers module attributes used for configuration, like `@mneme`.\n\n    * `auto_assert*` macros capture their arguments and environment in `%Mneme.Assertion{}` structs and output function calls that will run with the test and do the real heavy lifting.\n\n3. Tests begin to run. The Mneme server begins capturing all output from ExUnit, only flushing it to the console when the user is not being prompted.\n\n4. Auto-assertions run as tests are executed. Exactly what this means depends on the type of assertion (`auto_assert`, `auto_assert_raise`, etc.) and whether it is a new assertion with no reference value or not. The basic order of operations is:\n    * Capture the value being asserted and compare it against the reference value, if present. If it succeeds, carry on.\n\n    * If the assertion fails or there wasn't a reference value to begin with, a call is made to the Mneme server to update it.\n\n    * A set of patterns are constructed and presented to the user in diffs.\n\n    * The user either selects a pattern or rejects them. If selected, the update is stored for later and the test continues. If rejected, the test fails.\n\n5. Once all tests complete, the Mneme server updates the test files for any patched assertions and prints a summary.","ref":"architecture.html#overview"},{"type":"extras","title":"Generating assertions - Under the hood","doc":"TODO","ref":"architecture.html#generating-assertions"},{"type":"extras","title":"Server responsibilities - Under the hood","doc":"TODO","ref":"architecture.html#server-responsibilities"},{"type":"extras","title":"Terminal UI - Under the hood","doc":"TODO","ref":"architecture.html#terminal-ui"},{"type":"extras","title":"Diffing - Under the hood","doc":"TODO","ref":"architecture.html#diffing"}],"content_type":"text/markdown"}